-------------------------------------------------------------------------------
-- 3PointRotation.ms
-- By Sergo Pogosyan (sergo.pogosyan@gmail.com)
-- v 1.0.1
-- Created: 2006-12-04
-- tested using Max 7.0, 8.0
-- last edition: 2007-01-22
-------------------------------------------------------------------------------
-- Description: 
-- Script for rotating objects and groups by specifing 3 points. Like AutoCAD's 'rotate by reference' command
-------------------------------------------------------------------------------
-- Usage:
-- Click 3PointRotation button on the toolbar, script will ask you to enter base point.
-- Base point is the point about which rotating is performed. Second point is original point,
-- this is the point which you want to move to the new location by rotating this object.
-- And last, third point is reference point. Reference point shows angle of rotating.
-- After last point picked dialog box appears with checkbox "Flip", check this checkbox if object
-- rotated on the opposite direction.
-- 3PointRotate script behavior will be very familiar for the AutoCAD users - it is the analog
-- to the 'rotate by reference' operation
-- !for correct result 2.5d snap must be chosen!
-------------------------------------------------------------------------------
-- TO DO list:
-- Major features
-- 1. Add ability to check angle's direction to get rid of checkbox (comments are welcome)

-- Minor features
-- 1.
-------------------------------------------------------------------------------
-- history
-- 2007-01-22: Angle spinner added.
-- 2007-08-28: Bug fixed - in checking for good point content

--vars
basePoint = #escape
originalPoint = #escape
referencePoint = #escape
theAngle = 0

--rollouts
rotateDirection

--function
axisRotate 
axisRotateAbout 
axisParentRotateAbout

on execute do  
--i wanted to put this command to my 'root' script file. but it doesn't work.
--throwing exception saying "no local references permited here" or something like that.

(	
	
	-- dialog box's rollout.
	rollout rotateDirection "3Point Rotation"
	(	
		include "$scripts/SergoScripts/functions.ms"
		local ok_pressed = false
		local cancel_pressed = false
		local rotatedCW = false
		local rotatedCCW = false
		local cancel_rotation
		local temp_angle
		local obj_temp = excludeChildren selection as array
		
		fn cancel_rotation =
		(
			if rotatedCW then
			(
				rotateAngle = eulerangles 0 0 theAngle
				axisRotateAbout obj_temp rotateAngle basePoint
			)
			else if rotatedCCW do
			(
				rotateAngle = eulerangles 0 0 -theAngle
				axisRotateAbout obj_temp rotateAngle basePoint
			)
		)
		
		group "Direction"
		(
			checkbox direction "Flip" align:#center
			spinner angle_spinner "Angle" align:#center range:[-3600,3600,0]
			button cancel_but "Cancel" width:60 across:2
			button ok_but "Ok"	width:60 across:1
		)
		
		on rotateDirection open do
		(
			print "angle"
			print theAngle
			print obj_temp
			--rotate selection after rollout will be opened
			rotateAngle = eulerangles 0 0 theAngle
			axisRotateAbout obj_temp rotateAngle basePoint
			angle_spinner.value = theAngle
			temp_angle = theAngle
			rotatedCCW = true
			rotatedCW = false
		)
		
		on angle_spinner changed val do
		(				
			print "angle_delta"
			print -(temp_angle-val)
			if direction.checked then
				rotateAngle = eulerangles 0 0 (temp_angle-val)
			else
				rotateAngle = eulerangles 0 0 -(temp_angle-val)
			axisRotateAbout obj_temp rotateAngle basePoint
			temp_angle = val
			theAngle = val
			print "temp_angle"
			print temp_angle
		)
		
		on direction changed val do
		(
			-- set focus to the 'ok' button. I like to hit enter key after picking-and-clicking:)
			-- hope you'll enjoy
			setFocus rotateDirection.ok_but
			
			if not direction.checked then --if checkbox is unchecked - UNrotate selection
			(
				rotateAngle = eulerangles 0 0 (theAngle*2)
				axisRotateAbout obj_temp rotateAngle basePoint
				rotatedCCW = true
				rotatedCW = false
			)
			else if direction.checked then --
			(
				rotateAngle = eulerangles 0 0 -(theAngle*2)
				axisRotateAbout obj_temp rotateAngle basePoint
				rotatedCCW = false
				rotatedCW = true
			)
		)
		
		on cancel_but pressed do --on cancel button return selection to initial state
		(
			cancel_rotation()
			cancel_pressed = true
			destroyDialog rotateDirection --and close dialogbox
		)
		
		on ok_but pressed do
		(
			ok_pressed = true -- on ok button check the flag and close dialogbox
			destroyDialog rotateDirection
		)
		
		on rotateDirection close do
		(
			-- this is where flag setted in ok and cancel buttons listeners used
			-- if dialog box closing after ok or cancel button then do nothing.
			-- otherwise do the 'cancel' behavior
			if not ok_pressed and not cancel_pressed then
				cancel_rotation()
		)
	)
	-- picking point
	print ""
	basePoint = pickpoint snap:#3d prompt:"select base point"
	print basePoint
	if basePoint != #rightClick and basePoint != #escape then
	(
		originalPoint = pickpoint snap:#3d prompt:"select original point"
		print originalPoint
		if originalPoint != #rightClick and originalPoint != #escape then
		(
			referencePoint = pickpoint snap:#3d prompt:"select reference point"
			print referencePoint
			if referencePoint != #rightClick and referencePoint != #escape and referencePoint != undefined then
			(
				-- vectors
				v1 = originalPoint - basePoint
				v2 = referencePoint - basePoint
				
				--angle from super-function 'dot'
				theAngle = acos(dot (normalize v1) (normalize v2))
				
				createDialog rotateDirection modal:false escapeEnable:false
				setFocus rotateDirection.ok_but -- set focus to my lovely ok button.
			)
		)
	)	
)