-------------------------------------------------------------------------------
-- TransformationJitter.ms
-- By Sergo Pogosyan (sergo.pogosyan@gmail.com)
-- v 1.0
-- Created On: 10-02-2006
-- tested using Max 7.0, 8.0, 9.0
-- last edition: 2007-10-10
-------------------------------------------------------------------------------
-- Description:
-- Apply random transformations to selected objects
-------------------------------------------------------------------------------
-- Usage:
-- Enter transformations and press Apply button
-- Reset button resets all spinners to 0
-------------------------------------------------------------------------------
-- History:
-- 2007-10-10:	made scaling parameters relative to object's scale
--				added scaling in absolute values
--				added affect only parent objects
-------------------------------------------------------------------------------
-- Thanks to VG for adding scale axis locking
--


(
	global transJitter_floater

	rollout transJitter_mainRoll "Transformation Jitter"
	(
		--include "$scripts/SergoScripts/functions.ms"
		checkBox parents_only "Affect Only Parents" checked:true
		group "Position"
			(
			radiobuttons coordspos labels:#("World", "Local")
			spinner spnXMinPos "X: Min:" range:[0,100000,0] across:2 fieldWidth:40 offset:[-2,0]
			spinner spnXMaxPos "Max:" range:[0,100000,0] fieldWidth:40
			spinner spnYMinPos "Y: Min:" range:[0,100000,0] across:2 fieldWidth:40 offset:[-2,0]
			spinner spnYMaxPos "Max:" range:[0,100000,0] fieldWidth:40
			spinner spnZMinPos "Z: Min:" range:[0,100000,0] across:2 fieldWidth:40 offset:[-2,0]
			spinner spnZMaxPos "Max:" range:[0,100000,0] fieldWidth:40
			)
			
		group "Rotation"
			(
			spinner spnXRot "X:" width:80 range:[0,359,0] offset:[7,0]
			spinner spnYRot "Y:" width:80 range:[0,359,0] offset:[7,0]
			spinner spnZRot "Z:" width:80 range:[0,359,0] offset:[7,0]
			)
	
		group "Scale"
			(
			checkBox uniform_scale "Uniform Scale" checked:false across:2
			checkBox absolute_scale "Absolute" checked:false across:1 offset:[8,0]
			spinner spnXMinScale "X: Min:" range:[0,100000,0] across:2 fieldWidth:40 offset:[-2,0]
			spinner spnXMaxScale "Max:" range:[0,100000,0] fieldWidth:40
			spinner spnYMinScale "Y: Min:" range:[0,100000,0] across:2 fieldWidth:40 offset:[-2,0]
			spinner spnYMaxScale "Max:" range:[0,100000,0] fieldWidth:40
			spinner spnZMinScale "Z: Min:" range:[0,100000,0] across:2 fieldWidth:40 offset:[-2,0]
			spinner spnZMaxScale "Max:" range:[0,100000,0] fieldWidth:40
			)

			button btnApply "Apply" width:80
			button btnReset "Reset" width:80

		on uniform_scale changed state do
		(
			spnYMaxScale.enabled = not state
			spnZMaxScale.enabled = not state
			spnYMinScale.enabled = not state
			spnZMinScale.enabled = not state
		)
		
		on absolute_scale changed state do
		(
			if state == true then
			(
				if spnXMaxScale.value == 0 then
					spnXMaxScale.value = 100
				if spnYMaxScale.value == 0 then
					spnYMaxScale.value = 100
				if spnZMaxScale.value == 0 then
					spnZMaxScale.value = 100
				if spnXMinScale.value == 0 then
					spnXMinScale.value = 100
				if spnYMinScale.value == 0 then
					spnYMinScale.value = 100
				if spnZMinScale.value == 0 then
					spnZMinScale.value = 100
			)
			else
			(
				if spnXMaxScale.value == 100 then
					spnXMaxScale.value = 0
				if spnYMaxScale.value == 100 then
					spnYMaxScale.value = 0
				if spnZMaxScale.value == 100 then
					spnZMaxScale.value = 0
				if spnXMinScale.value == 100 then
					spnXMinScale.value = 0
				if spnYMinScale.value == 100 then
					spnYMinScale.value = 0
				if spnZMinScale.value == 100 then 
					spnZMinScale.value = 0

			)			
		)


		on btnApply pressed do
		(
			undo "Transformation Jitter" on
			(
				if parents_only.state then
					selection_array = sergo_excludeChildren selection as array
				else
					selection_array = selection as array
					
				for object in selection_array do
				(
					-- SCALING
					if uniform_scale.state then
					(
						uniScale = random -spnXMinScale.value spnXMaxScale.value
						if absolute_scale.state then
						(
							object.scale.x = abs(uniScale/100)
							object.scale.y = abs(uniScale/100)
							object.scale.z = abs(uniScale/100)

						)
						else
						(
							object.scale.x = abs(object.scale.x + ((object.scale.x/100)*uniScale))
							object.scale.y = abs(object.scale.y + ((object.scale.y/100)*uniScale))
							object.scale.z = abs(object.scale.z + ((object.scale.z/100)*uniScale))
						)
					)
					else
					(
						if absolute_scale.state then
						(
							object.scale.x = abs((random -spnXMinScale.value spnXMaxScale.value)/100)
							object.scale.y = abs((random -spnYMinScale.value spnYMaxScale.value)/100)
							object.scale.z = abs((random -spnZMinScale.value spnZMaxScale.value)/100)

						)
						else	
						(
							object.scale.x = abs(object.scale.x + ((object.scale.x/100)*(random -spnXMinScale.value spnXMaxScale.value)))
							object.scale.y = abs(object.scale.y + ((object.scale.y/100)*(random -spnYMinScale.value spnYMaxScale.value)))
							object.scale.z = abs(object.scale.z + ((object.scale.z/100)*(random -spnZMinScale.value spnZMaxScale.value)))
						)
					)
					-- SCALING

					-- POSITION
					if coordspos.state == 1 then
						in coordsys world
						(
							object.pos.x = object.pos.x + random -spnXMinPos.value spnXMaxPos.value
							object.pos.y = object.pos.y + random -spnYMinPos.value spnYMaxPos.value
							object.pos.z = object.pos.z + random -spnZMinPos.value spnZMaxPos.value
						)
					else
						in coordsys local
						(
							object.pos.x = (object.pos.x + random -spnXMinPos.value spnXMaxPos.value) - object.pos.x
							object.pos.y = (object.pos.y + random -spnYMinPos.value spnYMaxPos.value) - object.pos.y
							object.pos.z = (object.pos.z + random -spnZMinPos.value spnZMaxPos.value) - object.pos.z
						)
					-- POSITION

					-- ROTATION
					rotX = random -spnXRot.value spnXRot.value
					rotY = random -spnYRot.value spnYRot.value
					rotZ = random -spnZRot.value spnZRot.value

					rot = eulerangles rotX rotY rotZ

					rotate object rot
					-- ROTATION
				)
			)
		)

		on btnReset pressed do
		(
			spnXMinPos.value = spnXMaxPos.value = 0
			spnYMinPos.value = spnYMaxPos.value = 0
			spnZMinPos.value = spnZMaxPos.value = 0
			
			spnXRot.value = 0
			spnYRot.value = 0
			spnZRot.value = 0
			
			spnXMinScale.value = spnXMaxScale.value = 0
			spnYMinScale.value = spnYMaxScale.value = 0
			spnZMinScale.value = spnZMaxScale.value = 0
		)
	)

	rollout aboutRoll "About"
	(
		label lb_about "Transformation Jitter 1.01	"
		label lb_author "Sergo Pogosyan"
		hyperLink lb_email "sergo.pogosyan@gmail.com" address:"mailto:sergo.pogosyan@gmail.com" align:#center
		label lb_year "2007"
	)

	if transJitter_floater != undefined do CloseRolloutFloater transJitter_floater
	transJitter_floater = newRolloutFloater "Transformation Jitter" 210 435
	addRollout transJitter_mainRoll transJitter_floater
	addRollout aboutRoll transJitter_floater
	aboutRoll.open = false
)