-------------------------------------------------------------------------------
-- SergoScripts.mcr
-- MacroScripts File
-- By Sergo Pogosyan (sergo.pogosyan@gmail.com)
-- Created On: 2006-02-10
-- Last modification date: 2008-09-26
-------------------------------------------------------------------------------
-- 2008-09-26 Rotation on active axis added to rotateXXX commands
-- 2007-04-27 New 3PointScale script, with non-uniform scaling option
-- 2007-02-08 TapeCoorSys added
-- 2007-01-22 copy mode added to rotateXXX commands. Press shift to rotate the copy of selected object

macroScript TapeCoordSys
	category:"Sergo Pogosyan"
	toolTip:"Create Tape and select it as reference coordinate system"
	buttonText:"TapeCoordSys"
	Icon:#("sergoIcons",10)
(
	include "$scripts/SergoScripts/TapeCoordSys.ms"
)

macroScript BuildingMaker
	category:"Sergo Pogosyan" 
	toolTip:"Create Simple Buildings from the picked splines"
	buttonText:"Building Maker"
	Icon:#("sergoIcons",1)
(
	include "$scripts/SergoScripts/BuildingMaker.ms" 
)

macroScript StripeTexture
	category:"Sergo Pogosyan"
	toolTip:"Create bitmap with regular or random stripes"
	buttonText:"Stripe Texture"
	Icon:#("sergoIcons",2)
(
	include "$scripts/SergoScripts/StripeTexture.ms" 
)

macroScript WhiteVPBackground
	category:"Sergo Pogosyan"
	toolTip:"Make viewport background white"
	buttonText:"White Viewport"
	Icon:#("sergoIcons",3)
(
	include "$scripts/SergoScripts/WhiteVPBackground.ms"
)

macroScript TransformationJitter
	category:"Sergo Pogosyan"
	toolTip:"Apply random transformation to selected objects"
	buttonText:"Transformation Jitter"
	Icon:#("sergoIcons",4)
(
	include "$scripts/SergoScripts/TransformationJitter.ms"
)

macroScript RotateClockWise
	category:"Sergo Pogosyan"
	toolTip:"Rotate Object 90 degree on clockwise"
	buttonText:"Rotate 90 CW"
	Icon:#("sergoIcons",5)
(
	include "$scripts/SergoScripts/functions.ms"
	on isEnabled return
	(
		if (selection.count >= 1) and ((subobjectlevel==0)or(subobjectlevel==undefined )) then true	else false
	)
	
	on execute do
	(
		if (keyboard.shiftPressed) then 
		(
			print "shift pressed"
			maxops.clonenodes selection cloneType:#instance newNodes:&copyObjects
			selectedObjects = copyObjects
			select selectedObjects
		)
		else
		(
			print "shift not pressed"
			selectedObjects = getCurrentSelection()
		)
		objects = excludeChildren selection as array
		activeAxis = case toolmode.axisConstraints of
			(
				#x: 
				(
					rot90cw = eulerangles -90 0 0
					axisRotate objects rot90cw
				)
				#y: 
				(
					rot90cw = eulerangles 0 -90 0
					axisRotate objects rot90cw
				)
				#z: 
				(
					rot90cw = eulerangles 0 0 -90
					axisRotate objects rot90cw
				)
				default: messageBox "Activate one axis!" title:"Rotate ClockWise"
			)
	)
)

macroScript RotateCounterClockWise
	category:"Sergo Pogosyan"
	toolTip:"Rotate Object 90 degree on counterclockwise"
	buttonText:"Rotate 90 CCW"
	Icon:#("sergoIcons",6)
(
	include "$scripts/SergoScripts/functions.ms"
	on isEnabled return
	(
		if (selection.count >= 1) and ((subobjectlevel==0) or (subobjectlevel==undefined )) then true else false
	)
	on execute do
	(
		if (keyboard.shiftPressed) then 
		(
			maxops.clonenodes selection cloneType:#instance newNodes:&copyObjects
			selectedObjects = copyObjects
			select selectedObjects
		)
		else
		(
			selectedObjects = getCurrentSelection()
		)
		objects = excludeChildren selectedObjects
		activeAxis = case toolmode.axisConstraints of
					(
						#x: 
						(
							rot90ccw = eulerangles 90 0 0
							axisRotate objects rot90ccw
						)
						#y: 
						(
							rot90ccw = eulerangles 0 90 0
							axisRotate objects rot90ccw
						)
						#z: 
						(
							rot90ccw = eulerangles 0 0 90
							axisRotate objects rot90ccw
						)
						default: messageBox "Activate one axis!" title:"Rotate CounterClockWise"
					)
	)
)

macroScript RotateAround
	category:"Sergo Pogosyan"
	toolTip:"Rotate Object 180 degree"
	buttonText:"Rotate 180"
	Icon:#("sergoIcons",7)
(
	include "$scripts/SergoScripts/functions.ms"
	on isEnabled return
	(
		if (selection.count >= 1) and ((subobjectlevel==0) or (subobjectlevel==undefined )) then true else false
	)
	
	on execute do
	(
		if (keyboard.shiftPressed) then 
		(
			maxops.clonenodes selection cloneType:#instance newNodes:&copyObjects
			selectedObjects = copyObjects
			select selectedObjects
		)
		else
		(
			selectedObjects = getCurrentSelection()
		)
		objects = excludeChildren selection as array
		activeAxis = case toolmode.axisConstraints of
			(
				#x: 
				(
					rot180ccw = eulerangles 180 0 0
					axisRotate objects rot180ccw
				)
				#y: 
				(
					rot180ccw = eulerangles 0 180 0
					axisRotate objects rot180ccw
				)
				#z: 
				(
					rot180ccw = eulerangles 0 0 180
					axisRotate objects rot180ccw
				)
				default: messageBox "Activate one axis!" title:"Rotate Around"
			)
	)
)

macroScript Point3Scale
	category:"Sergo Pogosyan"
	toolTip:"Scale object by 3 points"
	buttonText:"3 Point Scale"
	Icon:#("sergoIcons",8)
(
	on isEnabled return
	(
		if (selection.count >= 1) and ((subobjectlevel==0) or (subobjectlevel==undefined )) then true else false
	)

	--on execute do
	--(
		include "$scripts/SergoScripts/3pointscale.ms"
	--)
)

macroScript Point3Rotate
	category:"Sergo Pogosyan"
	toolTip:"Rotate Selection by 3 points"
	buttonText:"3 Point Rotation"
	Icon:#("sergoIcons",9)
(
	on isEnabled return
	(
		if (selection.count >= 1) and ((subobjectlevel==0) or (subobjectlevel==undefined )) then true else false
	)

	include "$scripts/SergoScripts/3PointRotation.ms"
)